@@ -1,5 +1,4 @@
 package DateTime::Format::Pg;
-# $Id: /local/datetime/modules/DateTime-Format-Pg/trunk/lib/DateTime/Format/Pg.pm 11394 2007-05-22T11:20:32.433780Z daisuke  $
 
 use strict;
 use vars qw ($VERSION);
@@ -12,7 +11,7 @@ use DateTime::TimeZone 0.06;
 use DateTime::TimeZone::UTC;
 use DateTime::TimeZone::Floating;
 
-$VERSION = '0.16001';
+$VERSION = '0.16007';
 $VERSION = eval $VERSION;
 
 our @ISA = ('DateTime::Format::Builder');
@@ -226,8 +225,8 @@ my $pg_timeonly =
 #
 my $pg_datetime_iso =
 {
-  regex		=> qr/^(\d{4,})-(\d{2,})-(\d{2,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)?( BC)? *([-\+][\d:]+)?$/,
-  params 	=> [ qw( year    month    day      hour     minute  second nanosecond era    time_zone) ],
+  regex		=> qr/^(\d{4,})-(\d{2,})-(\d{2,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? *([-\+][\d:]+)?( BC)?$/,
+  params 	=> [ qw( year    month    day      hour     minute  second nanosecond time_zone       era) ],
   postprocess 	=> [ \&_fix_era, \&_fix_timezone, \&_fix_nanosecond ],
 };
 
@@ -235,8 +234,8 @@ my $pg_datetime_iso =
 #
 my $pg_datetime_pg_eu =
 {
-  regex		=> qr/^\S{3,} (\d{2,}) (\S{3,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? (\d{4,})( BC)? *((?:[-\+][\d:]+)|(?:\S+))?$/,
-  params 	=> [ qw(       day      month    hour     minute  second nanosecond year    era     time_zone) ],
+  regex		=> qr/^\S{3,} (\d{2,}) (\S{3,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? (\d{4,}) *((?:[-\+][\d:]+)|(?:\S+))?( BC)?$/,
+  params 	=> [ qw(       day      month    hour     minute  second nanosecond year      time_zone                 era ) ],
   postprocess 	=> [ \&_fix_era, \&_fix_timezone, \&_fix_nanosecond ],
 };
 
@@ -244,8 +243,8 @@ my $pg_datetime_pg_eu =
 #
 my $pg_datetime_pg_us =
 {
-  regex		=> qr/^\S{3,} (\S{3,}) (\s{2,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? (\d{4,})( BC)? *((?:[-\+][\d:]+)|(?:\S+))?$/,
-  params 	=> [ qw(       month    day      hour     minute  second nanosecond year    era     time_zone) ],
+  regex		=> qr/^\S{3,} (\S{3,}) (\s{2,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? (\d{4,}) *((?:[-\+][\d:]+)|(?:\S+))?( BC)?$/,
+  params 	=> [ qw(       month    day      hour     minute  second nanosecond year     time_zone                 era ) ],
   postprocess 	=> [ \&_fix_era, \&_fix_month_names, \&_fix_timezone, \&_fix_nanosecond ],
 };
 
@@ -254,8 +253,8 @@ my $pg_datetime_pg_us =
 #
 my $pg_datetime_sql =
 {
-  regex		=> qr/^(\d{2,})\/(\d{2,})\/(\d{4,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)?( BC)? *((?:[-\+][\d:]+)|(?:\S+))?$/,
-  params 	=> [ qw( month    day       year    hour     minute   second nanosecond era      time_zone) ],
+  regex		=> qr/^(\d{2,})\/(\d{2,})\/(\d{4,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? *((?:[-\+][\d:]+)|(?:\S+))?( BC)?$/,
+  params 	=> [ qw( month    day       year    hour     minute   second nanosecond    time_zone               era ) ],
   postprocess 	=> [ \&_fix_era, \&_fix_eu, \&_fix_timezone, \&_fix_nanosecond ],
 };
 
@@ -263,8 +262,8 @@ my $pg_datetime_sql =
 #
 my $pg_datetime_german =
 {
-  regex		=> qr/^(\d{2,})\.(\d{2,})\.(\d{4,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)?( BC)? *((?:[-\+][\d:]+)|(?:\S+))?$/,
-  params 	=> [ qw( day      month     year    hour     minute   second nanosecond era    time_zone) ],
+  regex		=> qr/^(\d{2,})\.(\d{2,})\.(\d{4,}) (\d{2,}):(\d{2,}):(\d{2,})(\.\d+)? *((?:[-\+][\d:]+)|(?:\S+))?( BC)?$/,
+  params 	=> [ qw( day      month     year    hour     minute   second nanosecond time_zone                 era ) ],
   postprocess 	=> [ \&_fix_era, \&_fix_timezone, \&_fix_nanosecond ],
 };
 
@@ -339,14 +338,24 @@ sub _fix_timezone {
   # Numerical time zone
   #
   
-  elsif($args{'parsed'}->{'time_zone'} =~ m/^[-\+](\d+)(:\d+)?$/) {
-    $args{'parsed'}->{'time_zone'} .= ':00' if !$2 && length($1) == 2;
+  elsif($args{'parsed'}->{'time_zone'} =~ m/^([-\+])(\d+)(?::(\d+))?(?::(\d+))?$/) {
+    my $tz;
+    if (length($2) == 2) {
+        # regular hour notation
+        my ($min, $sec) = ($3 || '00', $4 || '00');
+        $tz = sprintf "%s%02d:%02d:%02d", $1, $2, $min, $sec;
+    } else {
+        $tz = "$1$2";
+    }
+    $args{'parsed'}->{'time_zone'} = $tz;
   }
   
   # Non-numerical time zone returned, which can be ambiguous :(
   #
   else
   {
+    # XXX This barfs because 'self' may not necessarily be initialized
+    # Need to fix it
     my $stz = $args{'self'}->_server_tz($args{'args'} ? @{$args{'args'}} : ());
     $args{'parsed'}->{'time_zone'} = $stz || 'floating';
   }
@@ -373,7 +382,7 @@ DateTime::Format::Builder->create_class
   parsers =>
   {
     parse_date		=> [ $pg_dateonly_iso, $pg_dateonly_sql,
-    			     $pg_dateonly_german, ],
+    			     $pg_dateonly_german, $pg_infinity ],
     parse_timetz	=> [ $pg_timeonly, ],
     parse_timestamptz	=> [ $pg_datetime_iso, $pg_datetime_pg_eu,
                              $pg_datetime_pg_us, $pg_datetime_sql,
@@ -640,7 +649,7 @@ will contain the local time of the C<DateTime> object and no time zone.
 sub _format_fractional
 {
   my $ns = shift->nanosecond;
-  return $ns ? sprintf(".%09d", $ns) : ''
+  return $ns ? sprintf(".%09d", "$ns") : ''
 }
 
 sub format_time
@@ -693,7 +702,9 @@ C<DateTime> object.
 sub format_date
 {
   my ($self,$dt) = @_;
-  if($dt->year()<=0) {
+  if($dt->is_infinite) {
+    return $dt->isa('DateTime::Infinite::Future') ? 'infinity' : '-infinity';
+  } elsif($dt->year()<=0) {
     return sprintf('%04d-%02d-%02d BC',
       1-$dt->year(),
       $dt->month(),
@@ -882,13 +893,15 @@ list.  See http://lists.perl.org/ for more details.
 
 =head1 AUTHOR
 
-Claus A. Frber <perl@faerber.muc.de>
+Daisuke Maki E<lt>daisuke@endeworks.jpE<gt>
 
-Currently maintained by Daisuke Maki E<lt>daisuke@endeworks.jpE<gt>
+=head1 AUTHOR EMERITUS 
+
+Claus A. Faerber <perl@faerber.muc.de>
 
 =head1 COPYRIGHT
 
-Copyright  2003 Claus A. Frber. Copyright (c) 2005-2007 Daisuke Maki
+Copyright (c) 2003 Claus A. Faerber. Copyright (c) 2005-2007 Daisuke Maki
 
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
